

def get_human_readable_size(size, precision=2):
    if size == 0:
        return "0B"

    suffixes = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    index = 0
    while size > 1024 and index < len(suffixes):
        index += 1
        size = size / 1024

    return f"{size:.{precision}f} {suffixes[index]}"



import psutil

from .base import BaseCollection
from .enums import MetricsEnum


class CpuCollector(BaseCollection):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def event_type(self):
        return MetricsEnum.cpu.value

    @property
    def core_count(self):
        """Return the number of physical cores

        Returns:
            int: Total number of physical CPU cores
        """
        return psutil.cpu_count()

    @property
    def load_avg(self):
        """Return the average system load over the last 1, 5 and 15 minutes as a tuple.

        Returns:
            tuple: Average system load over the last 1, 5 and 15 minutes
        """
        return psutil.getloadavg()

    def to_dict(self):
        return {
            "headers": {
                "hostname": self.hostname,
                "ipaddress": self.ipaddress,
                "event_timestamp": self.event_timestamp,
                "event_type": self.event_type
            },
            "payload": {"cores": self.core_count, "load_avg": self.load_avg}
        }



import psutil

from .base import BaseCollection
from .enums import MetricsEnum
from .utils import get_human_readable_size


class MemoryCollector(BaseCollection):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def event_type(self):
        return MetricsEnum.disk.value

    @property
    def memory(self):
        return psutil.virtual_memory()

    @property
    def available(self):
        return get_human_readable_size(self.memory.available)

    @property
    def total(self):
        return get_human_readable_size(self.memory.total)

    @property
    def percentage(self):
        return self.memory.percent

    @property
    def used(self):
        return get_human_readable_size(self.memory.used)

    @property
    def free(self):
        return get_human_readable_size(self.memory.free)

    @property
    def active(self):
        return get_human_readable_size(self.memory.active)

    @property
    def inactive(self):
        return get_human_readable_size(self.memory.inactive)

    def to_dict(self):
        return {
            "headers": {
                "hostname": self.hostname,
                "ipaddress": self.ipaddress,
                "event_timestamp": self.event_timestamp,
                "event_type": self.event_type
            },
            "payload": {
                "available": self.available,
                "free": self.free,
                "used": self.used,
                "active": self.active,
                "inactive": self.inactive,
            }
        }



import psutil

from .base import BaseCollection
from .enums import MetricsEnum


class NetworkCollector(BaseCollection):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def event_type(self):
        return MetricsEnum.disk.value

    @property
    def io_counters(self):
        return psutil.net_io_counters()

    @property
    def bytes_sent(self):
        return self.io_counters.bytes_sent

    @property
    def bytes_recv(self):
        return self.io_counters.bytes_recv

    @property
    def packets_sent(self):
        return self.io_counters.packets_sent

    @property
    def packets_recv(self):
        return self.io_counters.packets_recv

    def to_dict(self):
        return {
            "headers": {
                "hostname": self.hostname,
                "ipaddress": self.ipaddress,
                "event_timestamp": self.event_timestamp,
                "event_type": self.event_type
            },
            "payload": {
                "bytes_sent": self.bytes_sent,
                "bytes_recv": self.bytes_recv,
                "packets_sent": self.packets_sent,
                "packets_recv": self.packets_recv,
            }
        }
